<?php

namespace Drupal\dblog_filter\Form;

use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Logger\RfcLogLevel;
use Drupal\Component\Render\FormattableMarkup;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * The DBLogFilterSettingsForm class.
 */
class DBLogFilterSettingsForm extends ConfigFormBase {

  /**
   * The module handler.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected $moduleHandler;

  /**
   * {@inheritdoc}
   */
  public function __construct(ModuleHandlerInterface $moduleHandler) {
    $this->moduleHandler = $moduleHandler;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('module_handler'),
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'dblog_filter_settings';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['dblog_filter.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config('dblog_filter.settings');
    $log_levels = RfcLogLevel::getLevels();
    foreach ($log_levels as $log_level) {
      $log_level_value = $log_level->getUntranslatedString();
      $severity_levels[strtolower($log_level_value)] = $log_level_value;
    }
    $moduleHandler = $this->moduleHandler;
    // Dblog settings.
    if ($moduleHandler->moduleExists('dblog')) {
      $form['dblog'] = [
        '#type' => 'details',
        '#open' => TRUE,
        '#title' => $this->t('DB Log'),
      ];
      $form['dblog']['severity_levels'] = [
        '#type' => 'checkboxes',
        '#title' => $this->t('Select Severity Levels'),
        '#description' => $this->t('Only the selected severity logs will be recorded'),
        '#options' => $severity_levels ?? [],
        '#default_value' => $config->get('severity_levels') ?? [],
      ];
      $form['dblog']['log_values'] = [
        '#type' => 'textarea',
        '#title' => $this->t('Enter DB Log type and Severity Levels.'),
        '#default_value' => $config->get('log_values'),
      ];
      $form['dblog']['method'] = [
        '#type' => 'radios',
        '#title' => $this->t("Method"),
        '#default_value' => $config->get('method') ?: 'include',
        '#options' => [
          'include' => $this->t('Only save selected'),
          'exclude' => $this->t('Save all but selected'),
        ],
      ];
      $log_values_description = new FormattableMarkup(
        '<i>Severity Levels: emergency, alert, critical, error, warning, notice, info, debug</i>
             <br/>
             <i>Log Types: php, cron, mail, mymodule, etc.</i>
             <p>
             @description
             <br/>
             <strong>
               @example_1
               <br/>
               @example_2
             </strong>
             </p>',
        [
          '@description' => 'Enter one value per line, in the format log_type|level1,level2,level3.. to restrict the log messages stored',
          '@example_1' => 'Example: php|error,notice,warning',
          '@example_2' => 'mymodule|error,notice',
        ]
      );
      $form['dblog']['description'] = [
        '#markup' => $this->t('@log_values_description', ['@log_values_description' => $log_values_description]),
      ];
    }
    // Syslog settings.
    if ($moduleHandler->moduleExists('syslog')) {
      $form['syslog'] = [
        '#type' => 'details',
        '#open' => TRUE,
        '#title' => $this->t('Syslog'),
      ];
      $form['syslog']['syslog_severity_levels'] = [
        '#type' => 'checkboxes',
        '#title' => $this->t('Select Severity Levels'),
        '#description' => $this->t('Only the selected severity logs will be recorded'),
        '#options' => $severity_levels ?? [],
        '#default_value' => $config->get('syslog_severity_levels') ?? [],
      ];
      $form['syslog']['syslog_method'] = [
        '#type' => 'radios',
        '#title' => $this->t("Method"),
        '#default_value' => $config->get('syslog_method') ?: 'include',
        '#options' => [
          'include' => $this->t('Only save selected'),
          'exclude' => $this->t('Save all but selected'),
        ],
      ];
    }
    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    // Save configurations.
    $values = $form_state->getValues();
    $config = $this->config('dblog_filter.settings');
    $fields = [
      'severity_levels',
      'log_values',
      'method',
      'syslog_severity_levels',
      'syslog_method',
    ];
    foreach ($fields as $field) {
      if (isset($values[$field])) {
        $config->set($field, $values[$field]);
      }
    }
    $config->save();
    parent::submitForm($form, $form_state);
  }

}
